@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2008 Sun Microsystems Inc.
+# Copyright 2004-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -33,22 +33,26 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py 3997 2008-06-23 17:18:28Z joanied $"
-__version__   = "$Revision: 3997 $"
-__date__      = "$Date: 2008-06-23 13:18:28 -0400 (Mon, 23 Jun 2008) $"
-__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import braillegenerator
+import pyatspi
+
+import braille_generator
 import debug
 import flat_review
+import formatting
 import keybindings
 import orca_state
 import settings
-import speechgenerator
+import speech_generator
 import structural_navigation
 import where_am_I
 import bookmarks
+import tutorialgenerator
 
 class Script:
     """The specific focus tracking scripts for applications.
@@ -70,7 +74,7 @@ class Script:
             self.name = "default"
 
         self.name += " (module=" + self.__module__ + ")"
-        
+
         self.listeners = self.getListeners()
 
         # By default, handle events for non-active applications.
@@ -82,14 +86,21 @@ class Script:
         self.pointOfReference = {}
         self.setupInputEventHandlers()
         self.keyBindings = self.getKeyBindings()
+        if settings.validateKeyBindings:
+            validation = self.keyBindings.validate()
+            if len(validation):
+                debug.println(debug.LEVEL_SEVERE, validation)
         self.brailleBindings = self.getBrailleBindings()
         self.app_pronunciation_dict = self.getPronunciations()
 
+        self.formatting = self.getFormatting()
         self.brailleGenerator = self.getBrailleGenerator()
         self.speechGenerator = self.getSpeechGenerator()
+        self.generatorCache = {}
         self.whereAmI = self.getWhereAmI()
         self.bookmarks = self.getBookmarks()
         self.voices = settings.voices
+        self.tutorialGenerator = self.getTutorialGenerator()
 
         self.flatReviewContextClass = flat_review.Context
 
@@ -176,15 +187,24 @@ class Script:
                 for command, handler in self.brailleBindings.iteritems()
                 if inputEventHandler == handler]
 
+    def getFormatting(self):
+        """Returns the formatting strings for this script."""
+        return formatting.Formatting(self)
+
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
-        return braillegenerator.BrailleGenerator(self)
+        return braille_generator.BrailleGenerator(self)
 
     def getSpeechGenerator(self):
         """Returns the speech generator for this script.
         """
-        return speechgenerator.SpeechGenerator(self)
+        return speech_generator.SpeechGenerator(self)
+
+    def getTutorialGenerator(self):
+        """Returns the tutorial generator for this script.
+        """
+        return tutorialgenerator.TutorialGenerator(self)
 
     def getEnabledStructuralNavigationTypes(self):
         """Returns a list of the structural navigation object types
@@ -211,10 +231,10 @@ class Script:
         """Returns the "where am I" class for this script.
         """
         return where_am_I.WhereAmI(self)
-    
+
     def echoKey(self, keyEvent):
         """Determine whether this script should echo the current key event.
-        Note that the keyEcho() method in orca.py will still take into 
+        Note that the keyEcho() method in orca.py will still take into
         account whatever the user's various preferences for key echoing
         are, which may override what is return by this echoKey() method.
 
@@ -231,7 +251,7 @@ class Script:
         """Returns the "bookmarks" class for this script.
         """
         try:
-            return self.bookmarks 
+            return self.bookmarks
         except AttributeError:
             self.bookmarks = bookmarks.Bookmarks(self)
             return self.bookmarks
@@ -323,10 +343,18 @@ class Script:
         #
         processEvent = (orca_state.activeScript == self \
                         or self.presentIfInactive)
+        if event.source.getRole() == pyatspi.ROLE_PROGRESS_BAR \
+           and not processEvent \
+           and settings.progressBarVerbosity == settings.PROGRESS_BAR_ALL:
+            processEvent = True
 
         if not processEvent:
             return
 
+        # Clear the generator cache for each event.
+        #
+        self.generatorCache = {}
+
         # This calls the first listener it finds whose key *begins with* or is
         # the same as the event.type.  The reason we do this is that the event
         # type in the listeners dictionary may not be as specific as the event
@@ -435,7 +463,7 @@ class Script:
         elif "default" in user_bindings_map:
             user_bindings = user_bindings_map["default"]
 
-        command = brailleEvent.event
+        command = brailleEvent.event["command"]
         consumes = False
         if user_bindings:
             consumes = command in user_bindings
@@ -474,7 +502,7 @@ class Script:
         #
         consumed = False
         user_bindings = None
-        command = brailleEvent.event
+        command = brailleEvent.event["command"]
 
         user_bindings_map = settings.brailleBindingsMap
         if self.name in user_bindings_map:
@@ -531,6 +559,17 @@ class Script:
         """
         pass
 
+    def isActivatableEvent(self, event):
+        """Returns True if the given event is one that should cause this
+        script to become the active script.  This is only a hint to
+        the focus tracking manager and it is not guaranteed this
+        request will be honored.  Note that by the time the focus
+        tracking manager calls this method, it thinks the script
+        should become active.  This is an opportunity for the script
+        to say it shouldn't.
+        """
+        return True
+
     def activate(self):
         """Called when this script is activated."""
         pass
